import { system, world, EntityComponentTypes, EquipmentSlot, GameMode, BlockPermutation, ItemComponentRegistry } from "@minecraft/server";

/**
 * Reduce la cantidad del ítem en la mano principal del jugador.
 * 
 * @param {Player} player - El jugador que usa el ítem.
 * @param {number} [quantity=1] - Cantidad a reducir (por defecto 1).
 */
function decrementStack(player, quantity = 1) {
    if (!player.isValid() || player.getGameMode() === GameMode.creative) return;
    
    const equippableComp = player.getComponent(EntityComponentTypes.Equippable);
    if (!equippableComp) return; // Verifica si el componente existe

    const item = equippableComp.getEquipment(EquipmentSlot.Mainhand);
    if (!item) return;

    if (item.amount <= quantity) {
        equippableComp.setEquipment(EquipmentSlot.Mainhand, null); // Elimina el ítem si es la última unidad.
    } else {
        item.amount -= quantity;
        equippableComp.setEquipment(EquipmentSlot.Mainhand, item);
    }
}

/**
 * Coloca un bloque flotante directamente sobre el agua, en el primer espacio de aire.
 * 
 * @param {Player} player - El jugador que coloca el bloque.
 * @param {BlockPermutation} permutationToPlace - El bloque que se colocará.
 * @param {Vector3} location - La ubicación inicial para buscar el lugar de colocación.
 * @returns {Block | undefined} - Devuelve el bloque colocado o undefined si no se pudo colocar.
 */
function placeBlockAboveWater(player, permutationToPlace, location) {
    let currentLocation = { ...location };
    const dimension = player.dimension;

    let block = dimension.getBlock(currentLocation);
    
    // Asegurar que la ubicación inicial sea agua o un bloque inundado
    if ((block.typeId === "minecraft:water" || block.isWaterlogged || block.hasTag("minecraft:water"))) {
        // Subimos hasta encontrar el primer bloque de aire
        while (currentLocation.y < 319) {
            currentLocation.y++; // Subimos una posición en el eje Y
            block = dimension.getBlock(currentLocation);

            // Si encontramos aire, colocamos el bloque aquí
            if (block.isAir) {
                block.setPermutation(permutationToPlace);
                decrementStack(player);
                return block;
            }
        }
    }

    return undefined; // Si no encontramos un lugar adecuado
}

// Mapa de correspondencia entre ítems y bloques
const itemToBlockMap = {
    "no:lilyred_item": "no:lilyred",
    "no:lilyblue_item": "no:lilyblue",
    "no:lilygreen_item": "no:lilygreen",
    "no:lilypurple_item": "no:lilypurple",
    "no:lilyrosa_item": "no:lilyrosa",
    "no:alga1_item": "no:alga1",
    "no:alga2_item": "no:alga2",
    "no:alga3_item": "no:alga3",
    "no:mininenufar_item": "no:mininenufar",
    "no:nenufar1_item": "no:nenufar1",
    "no:nenufar2_item": "no:nenufar2",
    "no:bignenufar_item": "no:bignenufar"
};

/**
 * Función para manejar el componente no:lilys.
 */
world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    console.log("Registrando el componente no:lilys");
    itemComponentRegistry.registerCustomComponent("no:lilys", {
        onUseOn(data) {
            console.log("Componente no:lilys activado");
            const { source, block } = data;

            if (block.typeId !== "minecraft:water" && !block.isWaterlogged) return;

            // Obtener el ítem en la mano del jugador
            const itemStack = source.getComponent(EntityComponentTypes.Equippable).getEquipment(EquipmentSlot.Mainhand);
            if (!itemStack) return;

            // Comprobar si el ítem tiene una correspondencia en el mapa
            const blockType = itemToBlockMap[itemStack.typeId];
            if (blockType) {
                // Colocar el bloque correspondiente sobre el agua
                placeBlockAboveWater(source, BlockPermutation.resolve(blockType), block.location);
            }
        }
    });
});

// Evento para `beforeEvents.itemUseOn`
world.beforeEvents.itemUseOn.subscribe((data) => {
    const { source, block, itemStack } = data;

    if (!data.isFirstEvent || (block.typeId !== "minecraft:water" && !block.isWaterlogged)) return;

    // Verificamos si el ítem existe y si tiene un bloque correspondiente
    if (itemStack) {
        // Comprobamos el bloque correspondiente al ítem
        const blockType = itemToBlockMap[itemStack.typeId];
        if (blockType) {
            data.cancel = true; // Evita el comportamiento predeterminado
            system.run(() => {
                // Colocamos el bloque flotante correspondiente sobre el agua
                placeBlockAboveWater(source, BlockPermutation.resolve(blockType), block.location);
            });
        }
    }
});